package com.qmth.teachcloud.common.threadPool;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;

import java.util.concurrent.Executor;
import java.util.concurrent.ThreadPoolExecutor;

/**
 * @Description: 计算线程池
 * @Author: CaoZixuan
 * @Date: 2022-03-10
 */
@Configuration
@EnableAsync
public class AnalyzeThreadExecutorConfig {
    private final static Logger log = LoggerFactory.getLogger(AnalyzeThreadExecutorConfig.class);
    private static final int cpuNum = Runtime.getRuntime().availableProcessors();

    @Value("${sys.config.threadPoolCoreSize}")
    Integer threadPoolCoreSize;

    @Value("${sys.config.analyzeThreadPoolCoreSize}")
    boolean customThreadPoolCoreSize;

    @Bean
    public Executor analyzeThreadPool(){
        log.info("start analyzeThreadPool");
        ThreadPoolTaskExecutor executor = new VisiableThreadPoolTaskExecutor();
        // 配置线程数
        if(! customThreadPoolCoreSize && cpuNum > 0){
            executor.setCorePoolSize(cpuNum + 1);
            executor.setMaxPoolSize(cpuNum * 2);
        }else {
            executor.setCorePoolSize(threadPoolCoreSize);
            executor.setMaxPoolSize(threadPoolCoreSize * 2);
        }
        //配置队列大小
        executor.setQueueCapacity(500);
        //配置线程池中的线程的名称前缀
        executor.setThreadNamePrefix("analyzeThreadPool-");
        // rejection-policy：当pool已经达到max size的时候，如何处理新任务
        // CALLER_RUNS：不在新线程中执行任务，而是有调用者所在的线程来执行
        executor.setRejectedExecutionHandler(new ThreadPoolExecutor.CallerRunsPolicy());
        //执行初始化
        executor.initialize();
        return executor;
    }
}
